import java.io.*;
import java.util.*;
import antlr.CommonAST;
import antlr.collections.AST;
import antlr.DumpASTVisitor;
import antlr.RecognitionException;
import antlr.TokenStreamException;

class ObjectClass implements Printable {

    String oid;
    String name;
    String description;
    Vector superclasses;
    boolean obsolete;
    Vector requiredAttributes;
    Vector optionalAttributes;

    public String toLdapV3Format() {
        return "objectclass ( " + oid + "\n" 
	    + ( ( name != null)? " NAME " + name : "")
	    + ( ( obsolete )?    " OBSOLETE " : "" )
            + ( ( description != null)? " DESC " + description : "")
            + ( ( superclasses != null)?  " SUP " 
		+ getAsLdapV3List(superclasses) : "")
	    + ( ( requiredAttributes != null)? " MUST " 
		+ getAsLdapV3List(requiredAttributes) : "")
	    + ( ( optionalAttributes != null)? " MAY " 
		+ getAsLdapV3List(optionalAttributes) : "")
            + " ) ";
    }

    /* objectclass <name> superior <superior objectclasses> requires <attrs> allows <attrs> 
       taken from netscaoe directory server.
    */
    public String toSlapdFormat() {
	return "# " + ( ( description != null)? " DESC " + description : "")
	    + "oid: " + oid 
	    + "\n objectclass " + name 
	    + ( (superclasses != null)? "\n    superior\n      "
		+ getVectorAsList(superclasses, ",\n      ") + "\n" : "") 
	    + ( (requiredAttributes != null)? "\n    requires\n      " 
		+ getVectorAsList(requiredAttributes, ",\n      ") : "")
	    + ( (optionalAttributes != null)? "\n    allows\n      " 
		+ getVectorAsList(optionalAttributes, ",\n      ") : "");
    }
   
    public void setName(String aName) { name = aName; }
    public void setDescription(String aDescr) { description = aDescr; }
    public void setObsolete(boolean obs) { obsolete = obs; }
    public void setOid(String aOid) { oid = aOid; }
    public void setSuperclasses(Vector superiors) { superclasses = superiors; }
    public void setRequiredAttributes(Vector reqAttrs) {
	requiredAttributes = reqAttrs;
    }
    public void setOptionalAttributes(Vector optAttrs) {
	optionalAttributes = optAttrs;
    }

    public static String getAsLdapV3List (Vector vec) {
	if (vec.size() == 1)
	    return (String) vec.firstElement();
	StringBuffer list = new StringBuffer("( ");
	Enumeration vecEnum = vec.elements();
	while (vecEnum.hasMoreElements()) {
	    list.append(vecEnum.nextElement());
	    if (vecEnum.hasMoreElements())
		list.append(" $ ");
	}
	list.append(" )");
	return list.toString(); 
    } 

    public static String getVectorAsList(Vector vec, String separator) {
	if (vec.size() == 1)
	    return (String) vec.firstElement();
	StringBuffer list = new StringBuffer("");
	Enumeration vecEnum = vec.elements();
	while (vecEnum.hasMoreElements()) {
	    list.append(vecEnum.nextElement());
	    if (vecEnum.hasMoreElements())
		list.append(separator);
	}
	return list.toString(); 
    }

}
