import java.io.*;
import java.util.Vector;
import java.util.Enumeration;
import antlr.CommonAST;
import antlr.collections.AST;
import antlr.DumpASTVisitor;
import antlr.RecognitionException;
import antlr.TokenStreamException;
/*
 * 11/17/2001 - 00:43:42
 *
 * SchemaConverter - converts ldap schema definitions
 * Copyright (C) 2001 Artur Schneider
 * mail@arturschneider.de
 * www.arturschneider.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
class SchemaConverter {

    public static void main(String[] args) throws java.io.IOException {
	SchemaConverter schemaConverter = 
	    new SchemaConverter(System.in, "ldapV3", 
				System.out, "slapd", System.err);
	schemaConverter.convert();
	
    }

    /** Stream for reading in schema */ 
    DataInputStream istream = null;
    /** Stream for printing converted schema.*/
    PrintStream out = null; 
    /** Stream for printing error messages */
    PrintStream err = null;
    String inputFormat = null;
    String outputFormat = null;

    SchemaConverter(InputStream in, String inputFormat, 
		    OutputStream out, String outputFormat,
		    OutputStream err) {
	istream = new DataInputStream(in);
	this.out = new PrintStream(out);
	this.err = new PrintStream(err);
	this.inputFormat = inputFormat;
	this.outputFormat = outputFormat;

	if (err == null) {
	    this.out.println ("No err given, using out instead");
	    this.err = this.out;
	}	


    }

    void convert() {
	if ("ldapV3".equals(inputFormat)
	    && "slapd".equals(outputFormat)) {
	    convertLdapV3ToSlapd();
	} else {
	    err.println ("  Conversion ( " + inputFormat 
			 + " -> " + outputFormat  + " ) not supported. ");
	}
    }

    void convertLdapV3ToSlapd() {
        LDAPv3SchemaLexer lexer = null;
        LDAPv3SchemaParser parser = null;
        CommonAST tree = null;
        try {
            lexer = new LDAPv3SchemaLexer(istream);
            lexer.setFilename("<stdin>");
	    parser = new LDAPv3SchemaParser(lexer);
            Vector vec = parser.schema();
	    printInSlapdFormat(vec);
	} catch (Exception e) {
	    err.println("Error in line " + lexer.getLine() 
                               +" column " + lexer.getColumn()  + "\n");
	    err.println(e);
	    e.printStackTrace();
	}	
    }

    public void printInLdapV3Format(Vector vec) {
        Enumeration defEnum = vec.elements();
	Printable def;
	while (defEnum.hasMoreElements()) {
            def = (Printable) defEnum.nextElement();
            out.println(def.toLdapV3Format());
	}
    }

    public void printInSlapdFormat(Vector vec) {
        Enumeration defEnum = vec.elements();
	Printable def;
	while (defEnum.hasMoreElements()) {
            def = (Printable) defEnum.nextElement();
            out.println(def.toSlapdFormat());
	}
    }
}
