header {
    import java.util.Vector;
    import java.util.HashMap;
}

/** 
 * This a grammar for LDAPv3 schema definitions
 * Taken from 
 *  "Understanding And Deploying LDAP Directory Services"
 *   Timothy A Howes, Ph.D. Mark C. Smith, Gordon S. Good
 *   Macmillan Technical Publishing, USA, 1999
 *
 * respectively rfc 2252
 */

class LDAPv3SchemaParser extends Parser ;
options {
    buildAST = true;
    k = 20;
}

schema returns [java.util.Vector vec]
  { 
    vec = new java.util.Vector();
    AttributeType at;
    ObjectClass   oc;
    
  }
    : (at=attributetype 
         {
           vec.add(at);
         }
      | oc=objectclass
         {
           vec.add(oc);
         } 
      )+
    ;

attributetype returns [AttributeType at]
  { 
    at=new AttributeType(); 
    String  oid=null;
    String  eqOid=null;
    String  orderingOid=null;
    String  substrOid=null;
  }
    :    "attributetype" LPAREN 
           (oid=oid)? 
            ( "NAME" name:QIDENT )?
            ( obsolete:"OBSOLETE" )?
            ( "DESC" desc:QIDENT )?
            ( "SUP"  sup:IDENT )?
            ( "EQUALITY" eqOid=oid )?
            ( "ORDERING" orderingOid=oid )?  
            ( "SUBSTR" substrOid=oid )?  
            ( "SYNTAX" )?
            ( "SINGLE-VALUE" )?
            ( "COLLECTIVE" )?
            ( "NO-USER-MODIFICATION" )?
            ( "USAGE" )?
         RPAREN
          {  
             at.setOid(oid);
             if (name != null) at.setName( name.getText());
             if (desc != null) at.setDescription( desc.getText());
             if (obsolete != null) at.setObsolete( true );
             at.setSuperclass( sup.getText());
             System.out.println("\n" + at.toLdapV3Format() + "\n"); 
          }
    ;


objectclass returns [ObjectClass oc]
  {
    oc = new ObjectClass();
    String oid = null;
  }
    : ("objectclass" | "objectClass" )  LPAREN
      oid=oid
      RPAREN
      {
        oc.setOid(oid);
      }
    ;

oid returns [String oid]
{ 
    String tmp = null;
    oid = "";
} 
    :   ( nr:INT ) 
        {
            oid  = nr.getText();
        }  
    |   ( o:INT DOT tmp=oid )
        {
            oid = o.getText() +"."+ tmp; 
        }
    ;




/*  LEXICAL RULES  */
class LDAPv3SchemaLexer extends Lexer;
options {
	//exportVocab=IDL;
	k=10;
   //analyzerDebug = true;
   //codeGenDebug = true;
}




//ATTRIBUTETYPE
//    : "attributetype"
//    ;

WS	
options {
  paraphrase = "white space";
}
	:	(' '
	|	'\t'
	|	'\n'  { newline(); }
	|	'\r')
		{ $setType(Token.SKIP); }
	;

DOT
    : '.'
    ;

LPAREN:	'('
	;

RPAREN:	')'
	;

//NAME : "NAME" ;


//SUP : "SUP" ;


QIDENT
	:	"'" IDENT "'"
	;



IDENT
options {
  testLiterals = true;
  paraphrase = "an identifer";
}
	:	('a'..'z'|'A'..'Z'|'_')('a'..'z'|'A'..'Z'|'_'|'0'..'9')* 
	;





protected
DIGIT
options {
  paraphrase = "a digit";
}
	:	'0'..'9'
	;

/* this lexical rule would result in an indeterministism between OID and INT 
OID
    : (INT)
    | (INT DOT OID)
    ;
*/

INT 
    : (o:DIGIT)+
    ;
